#include <iostream>

#include "cca.h"
#include "ports/Function_CCA.h"
#include "ports/RandomGenerator_CCA.h"
#include "ports/Measurement_CCA.h"

#include "MonteCarloIntegrator_CCA.h"
#include "macros_CCA.h"
#include <stdlib.h>
#include <unistd.h>

using namespace std;

namespace integrators
{
  namespace ccaimpl
  {
    // Integrator interface methods

    MonteCarloIntegrator::MonteCarloIntegrator ()
    {
      function_m = 0;
      random_m = 0;
      measurement_m = 0;
      frameworkServices = 0;
    }

    MonteCarloIntegrator::~MonteCarloIntegrator ()
    {
    }

    void MonteCarloIntegrator::foo(int x)
    {
      static performance::Timer * t1 = measurement_m->createTimer(
		      string("MonteCarloIntegrator::foo"),
		      string("void (int)"), string("Monte"));
        t1->start();
        t1->stop();
    }
    double MonteCarloIntegrator::integrate (double lowBound, double upBound,
                                            int count)
    {
      classic::gov::cca::Port * port;
      double sum = 0.0;

      // Get RandomGenerator port
      port = frameworkServices->getPort ("RandomGeneratorPort");
      if (port)
        random_m =
          dynamic_cast < randomgen::ccaports::RandomGenerator * >(port);
      if (random_m == 0)
        {
          cerr << "Connected to something other than a RandomGenerator port"
            << endl;
          return -1;
        }

      // Get Function port
      port = frameworkServices->getPort ("FunctionPort");
      if (port)
        function_m = dynamic_cast < functions::ccaports::Function * >(port);
      if (function_m == 0)
        {
          cerr << "Connected to something other than a Function port" << endl;
          return -1;
        }

      // Get Measurement port
      port = frameworkServices->getPort ("MeasurementPort");
      if (port)
        measurement_m = dynamic_cast < performance::ccaports::Measurement * >(port);
      if (measurement_m == 0)
        {
          cerr << "Connected to something other than a Measurement port" << endl;
          return -1;
        }

      static performance::Timer* top = measurement_m->createTimer(
		      	string("Top Level timer"));
      top->start();
      
      static performance::Timer* t = measurement_m->createTimer(
		      	string("IntegrateTimer"));

      cout <<"Before start!"<<endl;
      t->start();
      for (int i = 0; i < count; i++)
        {
          double x = random_m->getRandomNumber ();
          sum = sum + function_m->evaluate (x);
        }
      /* You can change the timer name... 
      string s = "NEW MonteCarloIntegrator's Integrate()"; 
      t->setName(s);
      */
      for (int i =0; i < 100000; i++)
        foo(2);

      /* Example of a user defined event */
      
      /*
      static performance::Event* evt = measurement_m->createEvent(string("Memory"));
      evt->trigger(1024);
      evt->trigger(2048);
      */

      /* Example of controlling instrumentation based on groups */
      /*
      static performance::Control * ctrl = measurement_m->createControl();
      ctrl->disableGroupName(string("Monte"));
      foo(23);
      */

      t->stop();

      static performance::Query* q = measurement_m->createQuery();
      const char **funcList;
      int numFuncs;
      q->getTimerNames(funcList, numFuncs);
      for(int i = 0; i < numFuncs; i++)
      { 
        cout <<"The function names are: "<<funcList[i]<<endl;
      }
      /* Now see the list of counters */
      const char **counterList;
      int numCounters;
      q->getCounterNames(counterList, numCounters);
      for(int i = 0; i < numCounters; i++)
      { 
        cout <<"The Counter names are: "<<counterList[i]<<endl;
      }

      /* Now get the metrics */
      double **counterExclusive;
      double **counterInclusive;
      int *numCalls;
      int *numChildCalls;
      const char **counterNames;
      int numberOfCounters;

      q->getTimerData(funcList, numFuncs, counterExclusive, counterInclusive,
		      numCalls, numChildCalls, counterNames, numberOfCounters);

      q->dumpTimerData(funcList, numFuncs);
      for(int i = 0; i < numFuncs; i++)
      {
	 cout << "Timer: "<<funcList[i]<< " NumCalls: "<<numCalls[i]
	      << " NumChildCalls: "<<numChildCalls[i]<<endl;
	 for (int cnt = 0; cnt < numberOfCounters; cnt++)
	 {
	   cout <<"Counter: "<<counterNames[cnt]
	        <<" Excl: "<<counterExclusive[i][cnt]
		<<" Incl: "<<counterInclusive[i][cnt]<<endl;
	 }
      }



/* CHECK */

      
      // STOP Top level timer
      top->stop(); 
      // Release ports
      frameworkServices->releasePort ("RandomGeneratorPort");
      frameworkServices->releasePort ("FunctionPort");
      frameworkServices->releasePort ("MeasurementPort");

      return (upBound - lowBound) * sum / count;
    }

    /**
     * The framework passes a pointer to the Services object
     * via this method, which is called by the framework as soon
     * as the component is instantiated.
     */
    void MonteCarloIntegrator::setServices (classic::gov::cca::Services * services)
    {
      if (services != 0)
        {
          frameworkServices = services;

          // Provide an Integrator port
	  classic::gov::cca::PortInfo * portInfo =
            frameworkServices->createPortInfo ("IntegratorPort",
                                               "integrators.ccaports.Integrator",
                                               0);
          if (portInfo != 0)
            frameworkServices->addProvidesPort (this, portInfo);

          // Use a Function port
          portInfo =
            frameworkServices->createPortInfo ("FunctionPort",
                                               "functions.ccaports.Function",
                                               0);
          frameworkServices->registerUsesPort (portInfo);

          // Use a RandomGenerator port
          portInfo =
            frameworkServices->createPortInfo ("RandomGeneratorPort",
                                               "randomgen.ccaports.RandomGenerator",
                                               0);
          frameworkServices->registerUsesPort (portInfo);

          // Use a MeasurementPort port
          portInfo =
            frameworkServices->createPortInfo ("MeasurementPort",
                                               "performance.ccaports.Measurement",
                                               0);
          frameworkServices->registerUsesPort (portInfo);
	  
        }
      else
        {
          // Close down if not closed already
          if (frameworkServices != 0)
            frameworkServices->removeProvidesPort ("IntegratorPort");
          frameworkServices = 0;
        }
    }

  }
}
