#include "cca.h"
#include <stdio.h>

#include "TauMeasurement_CCA.h"
#include "macros_CCA.h"
#include <iostream>
using namespace std;

#ifndef TAU_MPI_NOTDEFINED
#include <mpi.h>
#endif /* TAU_MPI_NOTDEFINED */

namespace performance 
{
    /** 
     * Default constructor
     */
     TauTimer::TauTimer(void)
     {
       string name = "noname";

       TAU_MAPPING_TIMER_CREATE(tautimer, name, " ", TAU_USER2, "CCA_DEFAULT");

     }

    /** 
     * constructor
     */
     TauTimer::TauTimer(string name)
     {

       TAU_MAPPING_TIMER_CREATE(tautimer, name, " ", TAU_USER2, "CCA_DEFAULT");
     }

    /** 
     * constructor
     */
     TauTimer::TauTimer(string name, string type)
     {

       TAU_MAPPING_TIMER_CREATE(tautimer, name, type, TAU_USER2, "CCA_DEFAULT");
     }

    /** 
     * constructor
     */
     TauTimer::TauTimer(string name, string type, string group)
     {

       TAU_MAPPING_TIMER_CREATE(tautimer, name, type, 
		       RtsLayer::getProfileGroup((char *)group.c_str()), 
		       group.c_str());
     }

    /** 
     * Default destructor
     */
    TauTimer::~TauTimer(void)
    {

    } 

    /** 
     * Start the timer
     */
    void TauTimer::start(void)
    {
      int tid = RtsLayer::myThread();
      TAU_MAPPING_PROFILE_TIMER(t, tautimer, tid);
      TAU_MAPPING_PROFILE_START(t, tid);
    } 

    /** 
     * Stop the timer
     */
    void TauTimer::stop(void)
    {
      TAU_MAPPING_PROFILE_STOP(RtsLayer::myThread());
    } 

    /** 
     * Set the name of the timer 
     */
    void TauTimer::setName(string name)
    {
      TAU_MAPPING_PROFILE_SET_NAME(tautimer, name);
    }

    /** 
     * Get the name of the timer 
     */
    string TauTimer::getName(void)
    {
      return TAU_MAPPING_PROFILE_GET_NAME(tautimer);
    }

    /** 
     * Set the type information of the timer 
     */
    void TauTimer::setType(string name)
    {
      TAU_MAPPING_PROFILE_SET_TYPE(tautimer, name);

    }

    /** 
     * Get the type information of the timer 
     */
    string TauTimer::getType(void)
    {

      return TAU_MAPPING_PROFILE_GET_TYPE(tautimer);
    }

    /** 
     * Set the group name of the timer 
     */
    void TauTimer::setGroupName(string name)
    {

      TAU_MAPPING_PROFILE_SET_GROUP_NAME(tautimer, name);
    }

    /** 
     * Get the group name of the timer 
     */
    string TauTimer::getGroupName(void)
    {
      return TAU_MAPPING_PROFILE_GET_GROUP_NAME(tautimer);

    }

    /** 
     * Set the group Id of the timer
     */
    void TauTimer::setGroupId(unsigned long group)
    {
      TAU_MAPPING_PROFILE_SET_GROUP(tautimer, group);
    }

    /** 
     * Get the group Id of the timer
     */
    unsigned long TauTimer::getGroupId(void)
    {
      return TAU_MAPPING_PROFILE_GET_GROUP(tautimer);
   
    }

    /** 
     * The implementation of the Query class. The Constructor.
     */
    TauQuery::TauQuery(void)
    {
    }

    /** 
     * Default destructor
     */
    TauQuery::~TauQuery(void)
    {
    }

    /** 
     * getTimerNames. Returns the list of timers and the number of timers
     */
    void TauQuery::getTimerNames(const char **& timerList, int& numTimers)
    {
      TAU_GET_FUNC_NAMES(timerList, numTimers);
    }

    /** 
     * getCounterNames. Returns the list of Counters and the number of Counters
     */
    void TauQuery::getCounterNames(const char **& counterList, int& numCounters)
    {
      TAU_GET_COUNTER_NAMES(counterList, numCounters);
    }

    /** 
     * getTimerData. Returns lists of metrics.
     */
    void TauQuery::getTimerData(const char **& inTimerList, int numTimers,
		    double **& counterExclusive, double **& counterInclusive,
		    int*& numCalls, int*& numChildCalls, 
		    const char **& counterNames, int& numCounters)
    {
      TAU_GET_FUNC_VALS(inTimerList, numTimers, counterExclusive, 
		      counterInclusive, numCalls, numChildCalls, 
		      counterNames, numCounters);
    }

    /** 
     * dumpProfileData. Writes the entire profile to disk in a dump file. 
     * It maintains a consistent state and represents the instantaneous 
     * profile data had the application terminated at the instance this call
     * is invoked.
     */
    void TauQuery::dumpProfileData(void)
    {
      TAU_DB_DUMP();
    }

    /**
     * dumpProfileDataPrefix. Writes the entire profile to disk in a dump
     * file prefixed by 'prefix'.  It maintains a consistent state and 
     * represents the instantaneous profile data had the application 
     * terminated at the instance this call is invoked.
     */
     void TauQuery::dumpProfileDataPrefix(const char *prefix) {
       TAU_DB_DUMP_PREFIX(const_cast<char*>(prefix));
     }

    /** 
     * dumpProfileDataIncremental. Writes the entire profile to disk in a 
     * dump file whose name contains the current timestamp. 
     * It maintains a consistent state and represents the instantaneous 
     * profile data had the application terminated at the instance this call
     * is invoked. This call allows us to build a set of timestamped profile
     * files. 
     */
    void TauQuery::dumpProfileDataIncremental(void)
    {
      TAU_DB_DUMP_INCR();
    }

    /** 
     * dumpTimerNames. Writes the list of timer names to a dump file on the 
     * disk.
     */
    void TauQuery::dumpTimerNames(void)
    {
      TAU_DUMP_FUNC_NAMES();
    }

    /** 
     * dumpTimerData. Writes the profile of the given set of timers to the 
     * disk. This allows the user to select the set of routines to dump and
     * periodically write the performance data of a subset of timers to disk
     * for monitoring purposes.
     */
    void TauQuery::dumpTimerData(const char **& inTimerList, int numTimers)
    {
      TAU_DUMP_FUNC_VALS(inTimerList, numTimers);
    }

    /** 
     * dumpTimerDataIncremental. Writes the profile of the given set of timers 
     * to the disk. The dump file name contains the current timestamp when the
     * data was dumped. This allows the user to select the set of routines to 
     * dump and periodically write the performance data of a subset of timers 
     * to the disk and maintain a timestamped set of values for post-mortem 
     * analysis on how the performance data varied for a given set of routimes
     * with time.
     */
    void TauQuery::dumpTimerDataIncremental(const char **& inTimerList, 
		    int numTimers)
    {
      TAU_DUMP_FUNC_VALS_INCR(inTimerList, numTimers);
    }

    /** 
     * User defined events default constructor
     */
    TauEvent::TauEvent()
    {
      event = new TauUserEvent("noname");
    }

    /** 
     * User defined events constructor
     */
    TauEvent::TauEvent(string name)
    {
      event = new TauUserEvent(name.c_str());
    }

    /** 
     * User defined events destructor
     */
    TauEvent::~TauEvent()
    {
    }
    
    /** 
     * Trigger a user defined event
     */
    void TauEvent::trigger(double data)
    {
      event->TriggerEvent(data);
    }

    /** 
     * TAU control instrumentation constructor
     */
    TauControl::TauControl()
    {
    }

    /** 
     * TAU control instrumentation destructor
     */
    TauControl::~TauControl()
    {
    }

    /** 
     * Control instrumentation. Enable group Id.
     */
    void TauControl::enableGroupId(unsigned long id)
    {
      TAU_ENABLE_GROUP(id);
    }
    
    /** 
     * Control instrumentation. Disable group Id.
     */
    void TauControl::disableGroupId(unsigned long id)
    {
      TAU_DISABLE_GROUP(id);
    }

    /** 
     * Control instrumentation. Enable group name.
     */
    void TauControl::enableGroupName(string name)
    {
      TAU_ENABLE_GROUP_NAME((char *)name.c_str());
    }
    
    /** 
     * Control instrumentation. Disable group name.
     */
    void TauControl::disableGroupName(string name)
    {
      TAU_DISABLE_GROUP_NAME((char *)name.c_str());
    }

    /** 
     * Control instrumentation. Enable all groups.
     */
    void TauControl::enableAllGroups(void)
    {
      TAU_ENABLE_ALL_GROUPS();
    }

    /** 
     * Control instrumentation. Disable all groups.
     */
    void TauControl::disableAllGroups(void)
    {
      TAU_DISABLE_ALL_GROUPS();
    }

  namespace ccaimpl 
  {
    /** 
     * Default constructor
     */
    TauMeasurement::TauMeasurement(void)
    {
      frameworkServices = 0;
    }

    /** 
     * Default destructor
     */
    TauMeasurement::~TauMeasurement(void)
    {
    }

    /** 
     * Create a timer
     */
    performance::Timer* TauMeasurement::createTimer(void) 
    {
      /**
       * Create a TauTimer instance and return it as a timer 
       */
      performance::Timer *timer = new performance::TauTimer();
      return timer;
    } // TauMeasurement::createTimer 

    performance::Timer* TauMeasurement::createTimer(string name) 
    {
      /**
       * Create a TauTimer instance and return it as a timer 
       */
      performance::Timer *timer = new performance::TauTimer(name);
      return timer;
    } // TauMeasurement::createTimer 

    performance::Timer* TauMeasurement::createTimer(string name, string type) 
    {
      /**
       * Create a TauTimer instance and return it as a timer 
       */
      performance::Timer *timer = new performance::TauTimer(name, type);
      return timer;
    } // TauMeasurement::createTimer 

    performance::Timer* TauMeasurement::createTimer(string name, string type, 
		    	string group) 
    {
      /**
       * Create a TauTimer instance and return it as a timer 
       */
      performance::Timer *timer = new performance::TauTimer(name, type, group);
      return timer;
    } // TauMeasurement::createTimer 

    performance::Query* TauMeasurement::createQuery(void) 
    {
      /**
       * Create a TauQuery instance
       */
      performance::Query *q = new performance::TauQuery();
      return q;
    }

    performance::Event* TauMeasurement::createEvent(void)
    {
      /**
       * Create a TauEvent instance and return it as an Event
       */
      performance::Event *event = new performance::TauEvent();
      return event;
    } // TauMeasurement::createEvent 

    performance::Event* TauMeasurement::createEvent(string name)
    {
      /**
       * Create a TauEvent instance and return it as an Event
       */
      performance::Event *event = new performance::TauEvent(name);
      return event;
    } // TauMeasurement::createEvent 

    performance::Control* TauMeasurement::createControl(void)
    {
      /**
       * Create a TauControl instance and return a pointer to Control
       */
      performance::Control *control = new performance::TauControl();
      return control;
    } // TauMeasurement::createControl 

    void tauSetNode(void)
    {
#ifndef TAU_MPI_NOTDEFINED 
// In the default case, MPI is defined. If you don't want this behavior,
// define TAU_MPI_NOTDEFINED 
//
      int flag, rank;
      MPI_Initialized(&flag);

      if (flag)
      {
	 PMPI_Comm_rank(MPI_COMM_WORLD, &rank);
	 TAU_PROFILE_SET_NODE(rank);
      }
      else
      {
	 cout <<"MPI has not been initialized yet"<<endl;
         TAU_PROFILE_SET_NODE(0);
      }
#else /* TAU_MPI_NOTDEFINED */
      cout <<"Inside tauSetNode(): Setting node id to 0"<<endl;
      TAU_PROFILE_SET_NODE(0);
#endif /* TAU_MPI_NOTDEFINED */

    }

    /** 
     * set the services provided by the TauMeasurement component
     */
    void TauMeasurement::setServices (classic::gov::cca::Services * services)
    {
      if (services != 0)
        {
          frameworkServices = services;

          // Provide a Measurement port
	  classic::gov::cca::PortInfo * portInfo =
            frameworkServices->createPortInfo ("MeasurementPort",
                                               "performance.ccaports.Measurement", 
                                               0);
          if (portInfo != 0)
            frameworkServices->addProvidesPort (this, portInfo);

	  // Set the node id of the MPI process in TAU
	  tauSetNode();
        }
      else
        {
          // Close down if not closed already
          if (frameworkServices)
            frameworkServices->removeProvidesPort ("MeasurementPort");
          frameworkServices = 0;
        }



    } // TauMeasurement::setServices 
  } /* namespace ccaimpl */
}
